import { BlockPermutation, world } from "@minecraft/server";

const cardinalSides = {
    north: { left: 'east', right: 'west' },
    south: { left: 'west', right: 'east' },
    west: { left: 'north', right: 'south' },
    east: { left: 'south', right: 'north' },
};

// Kapı açma ve kapama işlemi
const toggleDoor = (block, isOpen) => {
    block.setPermutation(block.permutation.withState('mcn:open', !isOpen));
    block.dimension.playSound(isOpen ? 'random.door_close' : 'random.door_open', block.location, { pitch: 1.0, volume: 0.4 });
};

// Kapının üst ve alt parçalarını al
const getHalfDoor = (block) => {
    return block.permutation.getState('mcn:section') === 'lower' ? block.above() : block.below();
};

// Kapı yerleştirme işlemi
const canPlaceDoor = (block, permutationToPlace) => {
    const aboveBlock = block.above();
    return aboveBlock.isAir || aboveBlock.isLiquid;
};

// Kapı yerleştirme yönlerini belirle
const getDoorSides = (block, permutationToPlace) => {
    const cardinalDirection = permutationToPlace.getState('minecraft:cardinal_direction');
    const sides = cardinalSides[cardinalDirection];
    const leftBlock = block[sides.left]();
    const rightBlock = block[sides.right]();
    return { leftBlock, rightBlock, cardinalDirection };
};

world.beforeEvents.worldInitialize.subscribe((data) => {
    // Kapı açma/kapanma bileşeni
    data.blockComponentRegistry.registerCustomComponent('mcn:toggle', {
        onPlayerInteract(event) {
            const { block } = event;
            const isOpen = block.permutation.getState('mcn:open');
            const halfDoor = getHalfDoor(block);

            toggleDoor(block, isOpen);
            if (halfDoor?.permutation.matches(block.typeId)) {
                toggleDoor(halfDoor, isOpen);
            }
        }
    });

    // Kapı yerleştirme bileşeni
    data.blockComponentRegistry.registerCustomComponent('mcn:on_place', {
        beforeOnPlayerPlace(event) {
            const { block, permutationToPlace } = event;
            if (!canPlaceDoor(block, permutationToPlace)) {
                event.cancel = true;
                return;
            }

            const { leftBlock, rightBlock, cardinalDirection } = getDoorSides(block, permutationToPlace);
            const isSideDoor = leftBlock.matches(permutationToPlace.type.id, { 'mcn:section': 'lower' }) && !rightBlock.matches(permutationToPlace.type.id, { 'mcn:section': 'lower' });
            const hasRightBlock = rightBlock.isSolid || rightBlock.above().isSolid;

            const doorStates = {
                'minecraft:cardinal_direction': cardinalDirection,
                'mcn:side': isSideDoor || hasRightBlock ? 'right' : 'left',
                'mcn:top_texture': event.player.isSneaking
            };

            const doorPermutation = BlockPermutation.resolve(permutationToPlace.type.id, doorStates);
            event.permutationToPlace = doorPermutation.withState('mcn:section', 'lower');
            block.above().setPermutation(doorPermutation.withState('mcn:section', 'upper'));
        }
    });

    // Değiştirilebilir doku bileşeni
    data.blockComponentRegistry.registerCustomComponent('mcn:changeable_texture', {
        beforeOnPlayerPlace(event) {
            const { player, permutationToPlace } = event;
            event.permutationToPlace = permutationToPlace.withState('mcn:top_texture', player.isSneaking);
        }
    });
});
